<?php

/**
 * @author Mariusz Chwalba <koder@mediasystems.pl>
 * @copyright 2009 Media Systems
 * @package cashbill
 *
 * Interface obsługi bramki SMS cashbill.
 *
 */

/**
 * 
 * Interface partnera do obsługi wiadomości SMS.
 * 
 * Dowolna liczba obiektów implementujących iSMSHandler może zostać dodana do 
 * cGateway przez metodę addSMSHandler(). Dla każdej odebranej wiadomości zostanie
 * wywołana metoda handle.
 * 
 * W przypadku błędu obsługi wiadomości oczekiwane jest przerwanie wykonywania
 * handlera przez wyjątek. Komunikat wyjątku zostanie przekazany do systemu Cashbill.
 * 
 * Wiadomość będzie transmitowana ponownie.
 *
 * W przypadku potrzeby zignorowania wiadomości (np. powtórnego przetworzenia
 * wiadomości o tym samym ID) metoda powinna zakończyć się normalnie.
 * 
 */
interface iSMSHandler
{
	public function handle( cMessageSMSIn $message );
}


/**
 * 
 * 
 * Inteface obsłużenia raportu dostarczenia wiadomości SMS.
 * 
 * Jeżeli usługa została skonfigurowania tak, by przesyłać szczątkowe raporty
 * o dostarczaniu wiadomości, to do serwera SOAP klienta zostaną wysyłane informacje
 * o statusie przetwarzania wiadomości.
 * 
 * Dowolna liczba obiektów implementujących iReportHandler może zostać dodana
 * do cGateway przez metodę addReportHandler. 
 *
 */
interface iReportHandler
{
	public function handle( cMessageReport $message );
}

/**
 * 
 * 
 * Interface partnera do obsługi błędów w transmisji wiadomości SMS.
 * 
 * Dowolna liczba obiektów implementujących iSMSErrorHandler może zostać dodana do 
 * cGateway przez metodę addSMSErrorHandler(). Dla każdej odebranej wiadomości zostanie
 * wywołana metoda handle.
 * 
 */
interface iSMSErrorHandler
{
	public function handle( cMessageSMSIn $message, $error );
}

/**
 * @internal Interface wiadomości, tylko do typehinting
 *
 */
interface iMessage
{
}

/**
 * @internal Klasa zawiera wspólne dla wszystkich wiadomości atrybuty.
 *
 */
abstract class aMessage implements iMessage
{
	/**
	 * Identyfikator wiadomości.
	 * 
	 * W przypadku wątpliwości co do statusu obsłużenia wiadomości zarówno partner jak i Cashbill
	 * może przesłać ponownie tą samą wiadomość. Jeżeli wiadomość została już wcześniej obsłużona
	 * poprawnie, ponowna transmisja powinna zostać zignorowana, z dowolnym raportem  
	 *
	 */
	public $id;
	/**
	 * Adres, pod który ma zostać dostarczona wiadomość.
	 * W przypadku wiadomości SMS/MMS oczekiwany jest numer telefonu z kodem kraju, np:
	 * 48xxxyyyzzz
	 *
	 */
	public $msisdn;
	
	/**
	 * Numer bramki Large Account, przez którą wiadomość została odebrana bądź zostanie wysłana.
	 *
	 */
	public $la;
	
	/**
	 * Treść przesłanej wiadomości.
	 * 
	 * Długość możliwej do przesłania treści zależna jest od medium transmisji.
	 * Maksymalna długość tego pola to 900 znaków. W przypadku przekroczenia
	 * limitu wiadomości SMS system wyśle wiele wiadomości, które zostaną złożone w całość
	 * w telefonie klienta
	 *
	 */
	public $text;
	
	/**
	 * Identyfikator usługi partnera.
	 *
	 */
	public $prefix;

	/**
	 * Czas dostarczenia wiadomości, w formacie unix, z mikrosekundami.
	 * W przypadku wiadomości wysyłanej może zostać pominięty - wtedy wiadomość
	 * zostanie wysłana od razu.
	 *
	 */
	public $time;
	
	/**
	 * Sygnatura wiadomości. Podpis MD5 najważniejszy parametr wiadomości,
	 * umożliwiający zweryfikowanie nadawcy.
	 * 
	 * Automatycznie sprawdzany przez metodę verifySign() dla wiadomości przychodzących
	 * Automatycznie tworzony przez metodę doSign() dla wiadomości wychodzących 
	 *
	 */
	public $sign;
	
	/**
	 * Bramka obsługująca wiadomość.
	 *
	 */
	protected $gateway;
	
	/**
	 * Przypisanie bramki obsługującej wiadomość.
	 * Metoda wywoływana automatycznie przez bramkę
	 * 
	 * @param cGateway $gateway
	 * @return aMessage
	 */
	public function setGateway( cGateway $gateway )
	{
		$this->gateway = $gateway;
		return $this;
	}
	
	/**
	 * @internal Podpisanie wiadomości odpowiednią sygnaturą. Metoda wykonywana automatycznie
	 * przez bramkę.
	 *
	 * @return aMessage
	 */
	public function doSign()
	{
		$this->sign = $this->computeSign();
		return $this;
	}
	
	/**
	 * @internal Sprawdzenie poprawności sygnatury. Metoda wykonywana automatycznie przez bramkę.
	 *
	 * @return boolean - true - gdy podpis jest poprawny
	 */
	public function verifySign()
	{
		return $this->sign == $this->computeSign();
	}

	/**
	 * @internal Oblicza sygnaturę wiadomości. Brak lub błędna sygnatura spowoduje odrzucenie
	 * wiadomości przez system Cashbill.
	 *
	 * @return string
	 */
	public function computeSign()
	{
		return md5(
		
				($this->id).
				($this->la).
				($this->getPrefix()).
				($this->gateway->privKey)
				
					);
	}
	
	/**
	 * @internal Podaje identyfikator usługi. W przypadku braku
	 * indywidualnego identyfikatora, podaje identyfikator z bramki.
	 *
	 */
	public function getPrefix()
	{
		return $this->prefix ? $this->prefix : $this->prefix = $this->gateway->defaultPrefix;
	}
	
}

/**
 * @page smsIn Odbieranie wiadomości SMS
 * 
 * Przesyłanie wiadomości SMS do systemu partnera odbywa się z wykorzystaniem
 * protokołu SOAP. Niniejszy pakiet instalacyjny zawiera gotowe oprogramowanie,
 * zwalniające partnera z konieczności przygotowania własnego klienta i serwera
 * SOAP, jak i gotowe szablony i pakiety realizujące większość typowych zadań
 * w obsłudze SMS.  
 * 
 * @section CBApiIn Obsługa wiadomości handlerem SMS
 * 
 * Zalecaną metodą obsługi wiadomości przychodząch jest przygotowanie własnej
 * klasy handlera wiadomości (implementującej iSMSHandler) i dodanie jej do
 * obieku bramki metodą addSMSHandler.
 * 
 * przykład:
 * 
 * @code
 * 
 * require_once 'cashbillSms.php';
 *  
 * class cMojaObslugaSMS implements iSMSHandler
 * {
 *    public function handle( cMessageSMSIn $sms )
 *    {
 *      // tutaj należy dopisać obsłużenie przychodzącej wiadomości SMS
 *    }
 * } 
 * 
 * $gateway = new cGateway( '<klucz tajny>', '<prefix>' );
 * $gateway->addSMSHandler( new cMojaObslugaSMS() );
 * $gateway->handleInput();
 * 
 * @endcode
 * 
 * @section CBApiInQueue Obsługa wiadomości kolejkami
 * 
 * Alternatywną metodą obsługi wiadomości SMS jest odczytanie ich bezpośrednio z
 * kolejki wiadomości bramki metodą receiveSMS. Każde wywołanie receiveSMS zwraca
 * koleją odebraną wiadomość lub \e null, gdy kolejka jest pusta. Obsługa wiadomości
 * sprowadza się do:
 *
 * @code
 * require_once 'cashbillSms.php';
 *  
 * $gateway = new cGateway( '<klucz tajny>', '<prefix>' );
 * $gateway->addSMSHandler( new cMojaObslugaSMS() );
 * $gateway->handleInput();
 * 
 * while( $sms = $gateway->receiveSMS() )
 * {
 * 	 // obsłużenie wiadomości SMS
 * }
 * @endcode
 * 
 * @section CBApiInPL Polskie znaki
 * 
 * Obecna wersja protokołu obsługuje polskie znaki diakrytyczne zarówno w wiadomościach
 * przychodzących jak i w wychodzących. Niniejsze API obsługuje kodowanie utf-8.
 *
 * Należy pamiętać, iż wysyłanie wiadomości z polskimi znakami powoduje zmianę kodowania
 * SMS skutkującą zmniejszeniem długości pojedynczego SMS ze 160 znaków do 80. Dla
 * wiadomości wieloczęściowych oznacza to skrócenie maksymalnego rozmiaru z 900 znaków
 * do 450. 
 */

/**
 * 
 * Wiadomość SMS przychodząca (wysłana przez klienta).
 * 
 * Przychodzące od klientów wiadomości są wysyłane do handlerów obsługi
 * wiadomości, dodanych do bramki przez addSMSHandler.
 * 
 * Handler wiadomości to szkieletowa klasa obejmująca jedynie metodę
 * handle, do której przekazywana jest otrzymana wiadomość.
 * 
 * Uruchomienie obsługi SMS przychodzących sprowadza się więc do utworzenia
 * własnego handlera i wpisania w metodzie handle odpowiedniej obsługi wiadomości.
 * 
 * Do bramki może zostać przypisane wiele handlerów. Każdy z nich otrzyma
 * przetwarzaną wiadomość. Taka konstrukcja umożliwia proste i szybkie dołączenie
 * do istniejącego oprogramowania klienta gotowych rozwiązań Cashbill, jak 
 * na przykład logowanie diagnostyczne. 
 *
 */
class cMessageSMSIn extends aMessage 
{
	/**
	 * Operator telefonii GSM, z usług którego korzysta klient
	 *
	 */
	public $net;

	/**
	 * Liczba części, z których składa się wiadomość. Wiadomości powyżej
	 * 160 znaków wysyłane są przez telefony komórkowe w częściach.
	 *
	 */
	public $numberOfParts;

	/**
	 * Liczba odebranych części wiadomości. W przypadku problemów w komunikacji
	 * GSM wiadomość może dotrzeć niekompletna. To pole pozwala na wykrycie tego
	 * problemu.
	 *
	 */
	public $recivedParts;
	
	/**
	 * Wygeneruj odpowiedź na tego SMSa
	 *
	 * @param string $text - treść odpowiedzi
	 * @param int    $nr   - numer odpowiedzi (opcjonalnie)
	 * @param int    $time - unix timestamp momentu, w którym powinna zostać dostarczona wiadomość (opcjonalnie)
	 * @return cMessageSMSOut
	 */
	public function reply( $text, $nr=1, $time=0 )
	{
		$reply = new cMessageSMSOut( $this->msisdn, $this->la, $text, $time, $this->getPrefix(), 'r.'.$nr.'.'.$this->id );
		
		$reply->setGateway( $this->gateway ); 
		return $reply;
	}

	/**
	 * Przygotowanie raportu 
	 *
	 * @param string $status - status, jaki chcemy raportować
	 * @param string $detail - tekst diagnostyczny (opcjonalne)
	 * @param float  $time   - unix timestamp momentu, w którym wystąpił status (opcjonalny) 
	 * @return cMessageReport
	 */
	public function makeReport( $status, $detail = '', $time = null )
	{
		$report = new cMessageReport();

		$report->id       = $this->id;
		$report->status   = $status;
		$report->time     = $time ? $time :  ( true );
		$report->errorstr = $detail;
		
		return $report;
	}
	
	/**
	 * Sprawdzenie, czy wiadomość jest kompletna - czy operator
	 * wysłał wszystkie części wiadomości (dla wiadomości powyżej 160 znaków).
	 * 
	 * Jeżeli wiadomość jest niekompletna, oznacza to, że pole text nie
	 * zawiera wszystkich znaków które wysłał użytkownik.
	 *
	 * @return boolean
	 */
	public function isCompelte()
	{
		return $this->numberOfParts == $this->recivedParts;
	}
}

/**
 * @page smsOut Wysyłanie wiadomości SMS
 * 
 * Przesyłanie wiadomości SMS do systemu Cashbill odbywa się z wykorzystaniem
 * protokołu SOAP. Niniejszy pakiet instalacyjny zawiera gotowe oprogramowanie,
 * zwalniające partnera z konieczności przygotowania własnego klienta i serwera
 * SOAP, jak i gotowe szablony i pakiety realizujące większość typowych zadań
 * w obsłudze SMS.  
 * 
 * @section CBApiOut Tworzenie i wysyłanie wiadomości
 * 
 * Zalecaną metodą obsługi wiadomości wychodzących jest przygotowanie nowego
 * obiektu cMessageSMSOut i wysłanie go poprzez metodę doSendMessageSMS bramki
 * cGateway.
 * 
 * przykład:
 * 
 * @code
 * 
 * 
 * require_once 'cashbillSms.php';
 *  
 * $gateway = new cGateway( '<klucz tajny>', '<prefix>' );
 * $sms = new cMessageSMSOut( '48xxxyyyzzz', '71880', 'Przykładowa wiadomość' );
 * $raport = $gateway->doSendMessageSMS( $sms );
 * 
 * @endcode
 * 
 */
/**
 * 
 * 
 * Wiadomość SMS wychodząca (odpowiedź do klienta)
 * 
 * Struktura zawiera kompletną informację o wiadomości wysyłanej
 * do klienta GSM.
 * 
 * Aby wysłać wiadomość do klienta należy utworzyć nową strukturę
 * przez new cMessageSMSOut( <parametry> ) i przekazać ją do bramki.
 * 
 * Dostępne opcje to:
 * 
 *   -# @code $sms->setGateway( $gateway )->send(); @endcode
 *   -# @code $gateway->doSendMessageSMS( $sms );   @endcode
 * 
 * gdzie:
 * 
 * $sms     - to nowo utworzona wiadomość cMessageSMSOut
 * $gateway - to zainicjowany obiekt cGateway
 *
 */
class cMessageSMSOut extends aMessage
{
	/**
	 * Pole pozwalające ustawić inny niż domyślne źródło SMS.
	 * Dostępne tylko w przypadku wiadomości marketingowych (płatnych od każdej odpowiedzi)
	 * 
	 * W przypadku odpowiedzi na SMS Premium Rate pole ignorowane.   
	 *
	 */
	public $laOverride;
	
	/**
	 * Identyfikator wiadomości, na którą odpowiedzią jest niniejsza.
	 *
	 * Opcjonalność tego pola jest zależna od podpisanej umowy.
	 * W przypadku usług o podwyższonym ryzyku system wymaga, by każda wysyłana odpowiedź
	 * zawierała identyfikator wiadomości, której dotyczy.
	 * 
	 * Nawet w przypadku gdy ta informacja jest opcjonalna zalecamy wypełnianie
	 * tego pola ze względu na ułatwienie i przyśpieszenie procesu reklamacyjnego. 
	 *
	 */
	public $replyId;
	
	/**
	 * Utworzenie nowej wiadomości. W przypadku odpowiadania na istniejącą wiadomość,
	 * należy skorzystać z metody reply() wiadomości źródłowej. Ustawia ona wszystkie
	 * konieczne pola, pozwalając na dodanie samej treści wiadomości zwrotnej
	 *
	 * @param string $msisdn   - numer telefonu adresata
	 * @param string $la       - numer bramki. usługa musi być uruchomiona na tej bramce
	 * @param string $text     - treść wiadomości do wysłania do klienta
	 * @param float $time      - moment wysłania wiadomości (unix timestamp). pominięcie spowoduje wysłanie natychmiastowe  
	 * @param string $prefix   - identyfikator (prefiks) usługi. Pominięcie spowoduje ustalenie domyślnego dla bramki.
	 * @param string $id       - unikalny identyfikator wiadomości. Pominięcie spowoduje wygenerowanie unikalnego identyfikatora.
	 * 
	 */
	public function __construct( $msisdn, $la, $text, $time=null, $prefix=null, $id = null )
	{
		$this->id     = $id ? $id : md5( $msisdn.$la.$text.$time );
		$this->msisdn = $msisdn;
		$this->la     = $la;
		$this->prefix = $prefix;
		$this->text   = $text;
		$this->time   = $time;
	}
	
	/**
	 * Wysyła tą wiadomość.
	 *
	 * @return cMessageSMSOut
	 */
	public function send()
	{
		$this->gateway->doSendMessageSMS( $this );
		return $this;
	}
	
	

}

/**
 * @page status Cykl życia wiadomości
 *
 * Każda wiadomość SMS przetwarzana przez system Cashbill przechodzi przez szereg operacji.
 * Zmiany statusu wiadomości przesyłane są do partnera w postaci raportów cMessageReport.
 * 
 * Aby odbierać i przetwarzać raporty o dostarczaniu wiadomości partner powinien utworzyć
 * klasę implementującą iReportHandler i dodać ją do bramki z wykorzystaniem addReportHandler.
 * Każdy cMessageReport zawiera identyfikator statusu, momentu wystąpienia oraz opis
 * diagnostyczny.
 *
 * W ramach niniejszego pakietu bramki dostarczony został gotowy handler obsługi raportów
 * cReportLogger, zapisujący wszystkie zmiany statusów wiadomości w pliku.
 *
 * Przykładowe przetwarzanie raportu:
 * 
 * @code
 * 
 * $gateway = new cGateway( '<klucz tajny>', '<prefiks>' );
 * $gateway->addReportHandler( new cReportLogger() );
 * $gateway->handleInput();
 * 
 * @endcode
 * 
 * W całym cyklu życia wiadomości statusy mogą występować w następującej kolejności
 * określonej poniższym grafem przejść.
 * 
 * Uwaga! Ze względu na transmitowanie danych o statusach przez publiczną sieć
 * internetową i związane z tym możliwe opóźnienia może dojść do sytuacji, gdy
 * wiadomości dotrą w innej kolejności. By ustalić prawdziwą koleność statusów
 * wiadomości należy posłużyć się polem określającym czas wystąpienia w cMessageReport.
 * 
 * Czasy zmiany statusów podawane są z dokładnością do mikrosekundy.
 * 
 * @dot
 * digraph statusDot
 * {
 *   node [shape=box, fontname=Helvetica, fontsize=10];
 *
 *   start  [ shape=ellipse ];
 *   stop   [ shape=ellipse label="sukces\n\nSMS dostarczony" color=green ];
 *
 *   start -> ACCEPTED;
 *   start -> NOT_ACCEPTED;
 *   
 *   ACCEPTED -> SENDING;
 *   SENDING  -> SENDING;
 *   SENDING  -> SEND_FAILED;
 *   SENDING  -> SEND_ERROR;
 *   SENDING  -> SENT;  
 *   
 *   SEND_ERROR -> SEND_FAILED;
 *   SEND_ERROR -> SEND_ERROR;
 *   SEND_ERROR -> SENDING;
 *   SEND_ERROR -> SENT
 *   
 *   SENT -> DELIVERING;
 *   DELIVERING -> DELIVERY_FAILED;
 *   DELIVERING -> DELIVERY_ERROR;
 *   DELIVERING -> DELIVERING;
 *   
 *   DELIVERING -> DELIVERED;
 *   
 *   DELIVERY_ERROR -> DELIVERY_ERROR;
 *   DELIVERY_ERROR -> DELIVERY_FAILED;
 *   DELIVERY_ERROR -> DELIVERING;
 *   DELIVERY_ERROR -> DELIVERED;
 *   
 *   DELIVERED -> stop;
 *   
 *   DELIVERY_FAILED [ color=red ];
 *   SEND_FAILED     [ color=red ];
 *   NOT_ACCEPTED    [ color=red ];
 *   
 * 	}
 *  @enddot
 */
	


/**
 * 
 * 
 * Raport z przetwarzania wiadomości
 *
 * Struktura zawiera informacje o procesie przetwarzania wiadomości w systemie
 * cashbill lub u partnera. Raport składa się ze statusu (jednej ze stałych
 * zadeklarowanych w ciele klasy), momentu wystąpienia i treści diagnostycznej.
 * 
 * Raporty mają głównie znaczenie diagnostyczne - pozwalają ustalić co się
 * dzieje z wiadomością po odebraniu przez infrastrukturę Cashbill, aż do dostarczenia
 * do klienta końcowego.
 * 
 */
class cMessageReport
{
	/**
	 * STATUS:
	 * Wiadomość zaakceptowana przez bramkę. Status wymagany.
	 * Wiadomości które nie miały tego statusu będą ponownie przesyłane przez system.
	 */
	const ACCEPTED 		    = 'ACCEPTED';
	/**
	 * STATUS:
	 * Wiadomość nie została przyjęta. Szczegóły w polu diagnostycznym.
	 */
	const NOT_ACCEPTED		= 'NOT_ACCEPTED';
	/**
	 * STATUS:
	 * Wiadomość w czasie transmitowania
	 */
	const SENDING		    = 'SENDING';
	/**
	 * STATUS:
	 * Wiadomość wysłana
	 */
	const SENT				= 'SENT';
	/**
	 * STATUS:
	 * Wystąpił błąd przy wysyłaniu wiadomości.
	 * Będzie przeprowadzona kolejna próba wysłania.
	 */
	const SEND_ERROR		= 'SEND_ERROR';
	/**
	 * STATUS:
	 * Wiadomość nie może być wysłana. Szczegóły błędu w polu diagnostycznym.
	 * Status finalny.
	 */
	const SEND_FAILED		= 'SEND_FAILED';
	/**
	 * STATUS:
	 * Wiadomość w trakcie transmisji w sieci GSM.
	 */
	const DELIVERING		= 'DELIVERING';
	/**
	 * STATUS:
	 * Wiadomość została dostarczona do klienta końcowego.
	 * Status finalny.
	 */
	const DELIVERED			= 'DELIVERED';
	/**
	 * STATUS:
	 * Błąd dostarczania wiadomości. Szczegóły w polu diagnostycznym.
	 * Wiadomość będzie automatycznie dostarczana ponownie.
	 */
	const DELIVERY_ERROR	= 'DELIVERY_ERROR';
	/**
	 * STATUS:
	 * Wiadomość nie może zostać dostarczona.
	 * Status finalny.
	 * 
	 */
	const DELIVERY_FAILED	= 'DELIVERY_FAILED';

	
	
	/**
	 * Identyfikator wiadomości, której dotyczy raport
	 *
	 */
	public $id;
	
	/**
	 * Status wiadomości. Stałe statusów zadeklarowane są w cMessageReport
	 *
	 */
	public $status;
	
	/**
	 * Opis diagnostyczny statusu, dostarczony przez podmiot przetwarzający
	 * wiadomość.
	 *
	 */
	public $errorstr;
	
	/**
	 * Moment wystąpienia statusu, w formacie unix timestamp, z mikrosekundami
	 *
	 */
	public $time;
	
	/**
	 * Liczba części, z których składa się przetwarzana wiadomość
	 *
	 */
	public $numberOfParts;
	
	/**
	 * Numer części, której dotyczy raport
	 *
	 */
	public $partNumber;
	
}

/**
 * Lista raportów.
 *
 * Zawiera w sobie listę raportów dotyczących wiadomości. Jest przekazywany
 * z systemu Cashbill, gdy konieczne jest przekazanie więcej niż jednego
 * raportu.
 * 
 * Raporty mogą dotyczyć jednej lub wielu wiadomości.
 *
 */
class cMessageReportList
{
	/**
	 * Tablica raportów
	 */
	public $report;
}

/**
 * Lista wiadomości wychodzących.
 * 
 * Zawiera wiele wiadomości wychodzących, które mogą zostać wysłane jednocześnie
 * w procesie rozsyłki masowej. Dostarcza również gotowych metod, dzięki którym można
 * łatwo przygotować masową rozsyłkę. 
 *
 */
class cMessageSMSOutList
{
	protected $gateway;
	public $message;
	
	/**
	 * Przygotowuje listę wiadomości. Ponieważ wszystkie wiadomości muszą być
	 * rozsyłane tą samą bramką, należy tą bramkę podać jako parametr konstruktora.
	 * 
	 * 
	 * @param cGateway $gateway - bramka, przez którą będą rozsyłane wiadomości.
	 */
	public function __construct( cGateway $gateway )
	{
		$this->gateway = $gateway;
	}
	
	/**
	 * Dodaje wiadomość do rozsyłki masowej.
	 * 
	 * @param cMessageSMSOut $msg - wiadomość do rozesłania
	 * @return cMessageSMSOutList
	 */
	public function add( cMessageSMSOut $msg )
	{
		$msg->setGateway( $this->gateway )->doSign();
		$this->message[] = $msg;
		
		return $this;
	}
	
	/**
	 *
	 * Przygotowuje wysyłkę wiadomości masowej (bulk).
	 * 
	 * @param $text      - treść wiadomości
	 * @param $msisdns   - tablica numerów telefonicznych odbiorców, w formacie (48xxxyyyzzz)
	 * @param $la        - bramka, z której ma być prowadzona rozsyłka
	 * @param $prefix    - identyfikator usługi, której dotyczy rozsyłka
	 * @param $time_from - czas rozpoczęcia rozsyłki (unix timestamp)
	 * @param $time_to   - czas zakończenia rozsyłki (unix timestamp)
	 * @return cMessageSMSOutList
	 */
	
	public function makeBulk( $text, $msisdns, $la, $prefix=null, $time_from=0, $time_to=0 )
	{
		if ( !$time_to ) $time_to = $time_from;
		
		$time_span = max( $time_to - $time_from, 0 );
		
		foreach( $msisdns as $msisdn )
		{
			$this->add( new cMessageSMSOut( $msisdn, $la, $text, $time_from + rand( 0, $time_span ), $prefix ) );
		}
		
		return $this;
	}
	
	/**
	 * Wysyła wszystkie wiadomości przez zadeklarowaną bramkę.
	 *  
	 * @return cMessageReportList
	 */
	public function send()
	{
		return $this->gateway->doSendMessagesSMS( $this );
	}
}

/**
 * Wyjątek bramki SMS  
 *
 */
class cGatewayException extends Exception
{
	/**
	 * Zignoruj wiadomość. Ten wyjątek nie powoduje zgłoszenia błędu,
	 * a jedynie zignorowanie wiadomości. Powinien być stosowany w handlerach
	 * obsługi unikalności identyfikatorów wiadomości.
	 * 
	 */
	const IGNORE_MESSAGE = 100;
	
	/**
	 * Prześlij tą wiadomość ponownie później.
	 */
	const RETRY_LATER = 200;
	
}

/**
 * 
 * Bramka SMS/MMS systemu Cashbill
 *
 * Przed zapoznaniem się ze szczegółową dokumentacją bramki zalecamy
 * obejrzenie schematów gotowych rozwiązań, zawartych w plikach
 * template_*.php.
 * 
 * Skorzystanie z gotowego schematu zapewnia najszybsze uruchomienie usługi.
 *
 */
class cGateway
{
	/**
	 * Adres WSDL serwera Cashbill służącego do wysyłania wiadomości.
	 *
	 */
	public $remoteWsdl = 'http://ssl.cashbill.pl/sms/gateway/smsBack/?wsdl';
	
	/**
	 * Adres WSDL serwera lokalnego, do którego Cashbill wysyła wiadomości
	 *
	 */
	public $localWsdl  = 'template_PartnerGateway.wsdl';
	
	/**
	 * URL na którym umieszczony jest serwer SOAP odbierający wiadomości
	 *
	 */
	public $mainUrl;
	
	/**
	 * Klucz prywatny bramki
	 *
	 */
	public $privKey;
	
	/**
	 * Domyślny identyfikator usługi, którym posługuje się bramka gdy nie został
	 * zdefiniowany indywidualny identyfikator usługi dla wychodzącej wiadomości
	 * SMS.
	 *
	 */
	public $defaultPrefix;
	
	/**
	 * Poziom cache WSDL klienta SOAP.
	 * 
	 * WSDL_CACHE_NONE
	 * WSDL_CACHE_DISK
	 * WSDL_CACHE_MEMORY
	 * WSDL_CACHE_BOTH 
	 *
	 * W czasie rozwoju aplikacji zalecamy stosowanie WSDL_CACHE_NONE.
	 * Po zakończeniu prac, by przyśpieszyć funkcjonowanie obsługi SMS,
	 * zalecane jest stosowanie WSDL_CACHE_MEMORY lub WSDL_CACHE_BOTH.
	 * 
	 */
	public $wsdlCache = WSDL_CACHE_BOTH;
	
	/**
	 * @internal Czy uruchomić śledzenie wywołań SOAP. Wymaga dodania
	 * kodu debugującego.
	 */
	public $wsdlTrace = false;
	
	protected $wsdlClassMap = array(
	
				'cMessageReport'     => 'cMessageReport',
				'cMessageSMSIn'      => 'cMessageSMSIn',
				'cMessageSMSOut'     => 'cMessageSMSOut',
				'cMessageReportList' => 'cMessageReportList',
				'cMessageSMSOutList' => 'cMessageSMSOutList',
	
			);
	
	protected $soap;
	
	protected $messages = array();
	protected $reports  = array();
	
	protected $SMSHandlers      = array();
	protected $SMSErrorHandlers = array();
	protected $reportHandlers   = array();
	
	/**
	 * Inicjalizacja obiektu bramki - odbiornika i nadajnika
	 * wiadomości SMS
	 *
	 * @param string $privKey       - Klucz prywatny bramki (do odczytania w panelu systemu, po uruchomieniu usługi)
	 * @param string $defaultPrefix - Domyślny identyfikator usługi bramki, przypisywany wiadomościom wychodzącym gdy nie został określony indywidualny identyfikator dla wiadomości 
	 * @param string $mainUrl       - Opcjonalny adres serwera SOAP odbierającego wiadomości. W przypadku pominięcia odczytywany jest z konfiguracji serwera. 
	 */
	public function __construct( $privKey, $defaultPrefix=null, $mainUrl = null )
	{
		$this->privKey = $privKey;
		$this->mainUrl = $mainUrl;
		
		$this->localWsdl = dirname( __FILE__ ).'/'.$this->localWsdl;
		
		$this->defaultPrefix = $defaultPrefix;
	}
	
	protected function getMainUrl()
	{
		return $this->mainUrl ? $this->mainUrl : $_SERVER['SCRIPT_URI'];
	}
	
	/**
	 * Obsłużenie requestu SOAP. Metodę należy wywoływać w serwerze SOAP odbierającym
	 * wiadomości z systemu Cashbill, którego URL został podany przez partnera w trakcie
	 * konfigurowania usługi. Metoda przechwytuje całą komunikację SOAP.
	 * 
	 */
	public function handleInput()
	{
		$url = $this->getMainUrl();
		
		if ( isset( $_GET['wsdl'] ) )
		{
			header( 'Content-Type: text/xml' );
			echo str_replace( '{location}', $url, file_get_contents( $this->localWsdl ) );
			die();
		}		

		try
		{
			$myServer = new SoapServer( $this->localWsdl, array( 'classmap' => $this->wsdlClassMap, 'cache_wsdl' => $this->wsdlCache, 'trace'=>1 ) );
			$myServer->setObject( $this );
			$myServer->handle();
		}
		catch ( Exception $e )
		{
			throw new SoapFault( $e->getMessage() );
		}
	}
	
	
	/**
	 * Dodanie handlera do obsługi wiadomości SMS.
	 *
	 * @param iSMSHandler $handler - klasa która będzie obsługiwała przychodzącą wiadomość SMS
	 * @return cGateway
	 */
	public function addSMSHandler( iSMSHandler $handler )
	{
		$this->SMSHandlers[] = $handler;
		return $this;
	}
	
	/**
	 * Dodanie handlera do obsługi błędów transmisji wiadomości SMS.
	 *
	 * @param iSMSErrorHandler $handler - obiekt który będzie obsługiwał błąd wiadomości SMS
	 * @return cGateway
	 */
	public function addSMSErrorHandler( iSMSErrorHandler $handler )
	{
		$this->SMSErrorHandlers[] = $handler;
		return $this;
	}
	/**
	 * Dodanie handlera do obsługi raportu.
	 *
	 * @param iReportHandler $handler - obiekt który będzie obsługiwał raporty o przetwarzaniu SMS
	 * @return cGateway
	 */
	public function addReportHandler( iReportHandler $handler )
	{
		$this->reportHandlers[] = $handler;
		return $this;
	}
	
	public function soap()
	{
		if ( $this->soap instanceof SoapClient ) return $this->soap;
		return $this->soap = new SoapClient( $this->remoteWsdl, array( 'classmap' => $this->wsdlClassMap, 'trace'=>$this->wsdlTrace, 'cache_wsdl' => $this->wsdlCache  ) );
	}
	
	/**
	 * Zwraca wszystkie raporty związane z wiadomością o podanym ID.
	 * 
	 * @param strign $id        - identyfikator wiadomości do sprawdzenia. 
	 * @return cMessageReport[] - tablica wszystkich raportów dotyczących wiadomości
	 * 
	 */
	public function doGetMessageReports( $id )
	{
		return $this->soap()->doGetMessageReports( $id );
	}
	
	/**
	 * Wysłanie wiadomości SMS poprzez bramkę Cashbill.
	 * 
	 * Jedną z metod wysłania nowej wiadomości SMS do klienta jest
	 * przygotowanie struktury cMessageSMSOut i przekazanie jej do
	 * metody doSendMessageSMS bramki.
	 * 
	 * @param cMessageSMSOut $SMS - wiadomość do wysłania
	 * @return cMessageReport     - raport o dostarczeniu wiadomości
	 */
	public function doSendMessageSMS( cMessageSMSOut $SMS )
	{
		$SMS->setGateway( $this );
		$SMS->doSign();
		
		$report = $this->soap()->doSendMessageSMS( $SMS );
		
		$this->doReceiveMessageReport( $report );
		
		return $report;
	}
	
	/**
	 * Wysyła do systemu listę wiadomości do wysłania. W przypadku
	 * rozsyłania dużej liczby wiadomości jest to zalecana metoda komunikacji
	 * z systemem Cashbill. Indywidualne wiadomości mogą mieć różne czasy
	 * dostarczenia. 
	 * 
	 * Faktyczna przepustowość rozsyłania wiadomości jest zależna od obciążenia
	 * systemu GSM operatora. W większości przypadków 
	 * 
	 * @param cMessageSMSOutList $msgList - lista wiadomości do rozesłania
	 * @return cReportList - lista raportów o przyjęciu wiadomości
	 */
	public function doSendMessagesSMS( cMessageSMSOutList $msgList )
	{
		$reports = $this->soap()->doSendMessagesSMS( $msgList );
		
		foreach( $reports->report as $report )
		{
			$this->doReceiveMessageReport( $report );
		}
		
		return $reports;
	}
	
	/**
	 * Wysyła do systemu Cashbill raport o statusie przetwarzania wiadomości.
	 * System Cashbill wymaga jedynie raportu "ACCEPTED". Wszystkie inne
	 * raporty posłużą jedynie w celach diagnostycznych.
	 *
	 * @param cMessageReport $report - raport do wysłania
	 * 
	 */
	public function doSendMessageReport( cMessageReport $report )
	{
		$this->soap()->doSentMessageReport( $report );
	}
	
	protected function handleError( cMessageSMSIn $SMS, $error )
	{
		foreach( $this->SMSErrorHandlers as $handler )
		{
			try 
			{
				$handler->handle( $SMS, $error );
			}
			catch ( Exception $e )	{ }
		}
	}
	
	/**
	 * @internal Metoda serwera SOAP, wywoływana w czasie komunikacji
	 */
	public function doReceiveMessageSMS( cMessageSMSIn $SMS )
	{
		$this->messages[] = $SMS;
		$SMS->setGateway( $this );
		
		if ( $SMS->verifySign() )
		{
			try
			{
				try 
				{
					foreach( $this->SMSHandlers as $handler )
					{
						$handler->handle( $SMS );
					}
				}
				catch ( cGatewayException $e )
				{
					if ( $e->getCode() == cGatewayException::IGNORE_MESSAGE )
					{
						$this->handleError( $SMS, 'Message ignored: '.$e->getMessage() );
						return $SMS->makeReport( cMessageReport::ACCEPTED, $e->getMessage() );
					}
					else
					{
						throw $e;
					}
				}
			}
			catch ( Exception $e )
			{
				$this->handleError( $SMS, 'Processing exception: '.$e->getMessage() );
				throw $e;
			}
			
			return $SMS->makeReport( cMessageReport::ACCEPTED );
		}
		else
		{
			$this->handleError( $SMS, 'Signature mismatch '.$SMS->sign.' vs '.$SMS->computeSign() );			
			return $SMS->makeReport( cMessageReport::SEND_ERROR, 'Signature mismatch' );			
		}
	}
	
	/**
	 * @internal Metoda serwera SOAP, wywoływana w czasie komunikacji
	 */
	public function doReceiveMessageReport( cMessageReport $report )
	{
		foreach( $this->reportHandlers as $handler )
		{
			$handler->handle( $report );
		}
		
		$this->reports[] = $report;
	}
	
	/**
	 * Zwraca pojedynczą odebraną wiadomość.
	 * 
	 * Alternatywną do handlerów metodą obsługi wiadomości jest skorzystanie z metody
	 * recieveSMS(), która zwraca kolejne wiadomości wysłane przez użytkowników.
	 * Ze względu na brak możliwości raportowania błędów zalecamy jednak by korzystać
	 * z handlerów.
	 * 
	 * @return cMessageSMSIn
	 */
	public function recieveSMS()
	{
		return array_shift( $this->messages );
	}
	
	/**
	 * Zwraca jeden odebrany raport
	 *
	 * Alternatywną do handlerów metodą obsługi raportów wiadomości jest skorzystanie z metody
	 * recieveReport(), która zwraca kolejne odebrane raporty o dostarczanych wiadomościach. 
	 * Ze względu na brak możliwości raportowania błędów zalecamy jednak by korzystać
	 * z handlerów.
	 * 
	 * @return cMessageReport
	 */
	public function recieveReport()
	{
		return array_shift( $this->reports );
	}
}


?>